/*
    Copyright (C) 2003 Pedro Lamarao Rosa e Silva
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef IRC_MSG_H
#define IRC_MSG_H 1

#include <string>
#include <list>
#include <istream>
#include <ostream>

class irc_msg_prefix
{
  public:

    friend class irc_msg;

    /**
     * Default constructor.
     */
    irc_msg_prefix ()
    : _M_is_empty(true)
    { ; }

    /**
     * Constructor.
     */
    irc_msg_prefix (const std::string& s)
    : _M_is_empty(true)
    { this->parse(s); }

    /**
     * Prefix nickname.
     */
    const std::string&
    nick () const
    { return _M_nick; }

    /**
     * Prefix username.
     */
    const std::string&
    user () const
    { return _M_user; }

    /**
     * Prefix hostname.
     */
    const std::string&
    host () const
    { return _M_host; }

    /**
     * If prefix is empty or not.
     */
    bool
    empty () const
    { return _M_is_empty; }

  protected:

    /**
     * Parse std::string as a IRC message prefix.
     */
    void
    parse (const std::string& s);

    void
    clear ()
    {
      _M_nick.clear();
      _M_user.clear();
      _M_host.clear();
      _M_is_empty = true;
    }

  private:
    std::string _M_nick;
    std::string _M_user;
    std::string _M_host;
    bool        _M_is_empty;
};

class irc_msg
{
  public:

    typedef std::list<std::string> args_t;

    /**
     * Constructor.
     */
    irc_msg ()
    { ; }

    /**
     * Constructor.
     */
    irc_msg (const std::string& s)
    { this->parse(s); }

    /**
     * IRC message prefix.
     */
    const irc_msg_prefix&
    prefix () const
    { return _M_prefix; }

    /**
     * IRC message command.
     */
    const std::string&
    cmd () const
    { return _M_cmd; }

    /**
     * IRC message arguments.
     */
    const args_t&
    args () const
    { return _M_args; }

    /**
     * IRC message nth argument.
     */
    const std::string&
    arg (int n) const
    {
      args_t::const_iterator it = _M_args.begin();
      for (int i = 1; i < n && it != _M_args.end() ; i++)
        it++;
      return *it;
    }

    /**
     * IRC message text data.
     */
    const std::string&
    text () const
    { return _M_text; }

    friend std::istream&
    operator>> (std::istream& is, irc_msg& ircm);

    friend std::ostream&
    operator<< (std::ostream& os, const irc_msg& ircm);

  protected:

    /**
     * Parse std::string as a IRC message.
     */
    void
    parse (const std::string& s);

    void
    clear ()
    {
      _M_prefix.clear();
      _M_cmd.clear();
      _M_args.clear();
      _M_text.clear();
    }

  private:
    irc_msg_prefix    _M_prefix;
    std::string       _M_cmd;
    args_t            _M_args;
    std::string       _M_text;
};

#endif // IRC_MSG_H
