// -*- Socket Streams -*-

// Copyright (C) 2003 Pedro Lamarao <pedro.lamarao@izcom.com.br>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file address_factory.h
 * @brief Contains the address class.
 */

#ifndef SOCKETSTREAM_ADDRESS_INTERFACE_H
#define SOCKETSTREAM_ADDRESS_INTERFACE_H 1

#include <socketstream/sockets_base.h>
#include <socketstream/address_exception.h> // for class address_exception

namespace socketstream {

  /**
   * @brief Interface for address classes.
   * 
   * Implementations of the address class deal with specific address families.
   * Not all address families have "name resolution".
   */
  class address
  {
  public:

    /**
     * @brief Destructor.
     * 
     * Does nothing special.
     */
    virtual
    ~address() { }

    /**
     * @brief Get stored address.
     * 
     * @return Address.
     */
    const sockaddr*
    getaddr () { return this->resolve(); }

    /**
     * @brief Get size of stored address data.
     * 
     * @return Size.
     */
    socklen_t
    getsize () { return this->size(); }

    /**
     * @brief Request address for specific socket style.
     * 
     * @param style Socket style.
     * @return *this
     */
    virtual address&
    style (sockets_base::style __style) { return *this; }

    /**
     * @brief Request address for a bind.
     * 
     * @return *this
     */
    virtual address&
    passive () { return *this; }

    /**
     * @brief Don't do name lookup.
     * 
     * @return *this
     */
    virtual address&
    numeric () { return *this; }

  protected:

    /**
     * @brief Resolve address.
     * 
     * @return Address data.
     * 
     * This function do "name resolution" if needed.
     * This may be a costly operation.
     */
    virtual sockaddr*
    resolve () throw (address_exception) { return 0; }

    /**
     * @brief Get size of address data.
     * 
     * @return Size.
     */
    virtual socklen_t
    size () { return 0; }

  };
}

#endif // SOCKETSTREAM_ADDRESS_INTERFACE_H
