// -*- Socket Streams -*-

// Copyright (C) 2003 Pedro Lamarao <pedro.lamarao@izcom.com.br>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file basic_isocketstream.h
 * @brief Contains the isocketstream template.
 */

#ifndef SOCKETSTREAM_BASIC_ISOCKETSTREAM_H
#define SOCKETSTREAM_BASIC_ISOCKETSTREAM_H 1

#include <istream>
#include <socketstream/basic_socketbuf.h>

namespace socketstream {

  /**
   * @brief Controlling input on sockets.
   */
  template <typename _CharT, typename _Traits>
  class basic_isocketstream
  : public std::basic_istream<_CharT, _Traits>,
    public sockets_base
  {
  public:

    //@{
    /**
     * @brief These are standard types.
     * 
     * They permit a standardized way of referring to names of
     * (or names dependant on) the template parameters, which are specific 
     * to the implementation.
     */
    typedef _CharT                    char_type;
    typedef _Traits                   traits_type;

    typedef typename traits_type::int_type int_type;
    typedef typename traits_type::pos_type pos_type;
    typedef typename traits_type::off_type off_type;
    //@}

    typedef basic_socketbuf<char_type, traits_type> __socketbuf_type;
    typedef std::basic_istream<char_type, traits_type> __stream_type;

    /**
     * @brief Default constructor.
     */
    basic_isocketstream()
    : __stream_type(&_M_buffer), _M_buffer()
    { }

    /**
     * @brief Constructor.
     * 
     * @param f Socket family.
     * @param s Socket style.
     */
    basic_isocketstream(family __f, style __s)
    : __stream_type(&_M_buffer), _M_buffer(__f, __s)
    { }

    /**
     * @brief Constructor.
     * 
     * @param s Socket.
     * 
     * Creates a stream over a copy of the given socket.
     */
    basic_isocketstream (const basic_socket& __s)
    : __stream_type(&_M_buffer), _M_buffer(__s)
    { }

    /**
     * @brief Destructor.
     * 
     * Does nothing special.
     */
    virtual
    ~basic_isocketstream() { }

    /**
     * @brief Checks if stream is open.
     * 
     * @return true is stream is open, false otherwise.
     */
    bool
    is_open ()
    { return _M_buffer.is_open(); }

    /**
     * @brief Opens a connection.
     * 
     * @param addr Address of peer.
     */
    void
    open (address& __addr)
    {
      if (!_M_buffer.open(__addr))
        this->setstate(std::ios_base::failbit);
    }

    /**
     * @brief Shuts stream down.
     * 
     * @param mode Shutdown mode.
     * 
     * This effectively closes the connection, as opposed
     * to closing the stream on the system (which is attempted with
     * a close() ).
     */
    void
    shutdown (shutmode __mode)
    {
      if (!_M_buffer.shutdown(__mode))
        this->setstate(std::ios_base::failbit);
    }

    __stream_type&
    operator<< (io_flags& __flag)
    {
      _M_buffer.setf(__flag);
      return *this;
    }
    
  private:

    __socketbuf_type _M_buffer;

  };

  typedef basic_isocketstream<char, std::char_traits<char> > isocketstream;

}

#endif	// SOCKETSTREAM_BASIC_ISOCKETSTREAM_H
