// -*- Socket Streams -*-

// Copyright (C) 2003 Pedro Lamarao <pedro.lamarao@izcom.com.br>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file basic_socket.h
 * @brief Contains the socket class.
 */

#ifndef SOCKETSTREAM_BASIC_SOCKET_H
#define SOCKETSTREAM_BASIC_SOCKET_H 1

#include <sys/types.h>
#include <sys/socket.h>
#include <socketstream/sockets_base.h>
#include <socketstream/socket_exception.h>
#include <socketstream/address.h>

namespace socketstream {
  /**
   * @brief Socket class wraps up the C socket interface.
   * 
   * This class implements "resource acquisition is initialization" for
   * socket descriptors.
   *
   * The purpose of this class is to merely wrap the C socket API.
   * Thus, it's member function prototypes follow closely those of the C
   * API, and no error treatment other than the trivial is done.
   *
   * The only big departure from the C API is the fact that error conditions
   * intercepted are thrown as socket_exception objects containing the
   * error code from std::errno. Thus, almost all functions now return void.
   */
  class basic_socket
  {
  public:

    /**
     * @brief This is the type of a socket descriptor on the system.
     * 
     * This is usually int, but don't count on it unless you're Unix-only.
     */
    typedef sockets_base::socket_type socket_type;

    /**
     * @brief Default constructor.
     * Socket's constructor wraps the socket() system call,
     * which allocates a socket descriptor in the system for this process.
     * This constructor will create a socket of default family and style,
     * currently sockets_base::inet and sockets_base::stream.
     */
    basic_socket ();

    /**
     * @brief Constructor.
     * 
     * @param family Socket's address family.
     * @param style Socket's communication style.
     * @param protocol Socket's communication protocol.
     * Socket's constructor wraps the socket() system call,
     * which allocates a socket descriptor in the system for this process.
     * 
     * Constructs socket of specified family, style and protocol.
     * We usually use a default protocol.
     */
    basic_socket (int __family, int __style, int __protocol = 0);

    /**
     * @brief Constructor for an existing socket descriptor.
     * @param sd Socket descriptor.
     * @param close If true, close this socket on destruction.
     * 
     * This constructor does no call socket(), and just associates itself
     * with the specified socket descriptor.
     * Currently, we lose family, style and protocol information.
     */
    basic_socket (socket_type __sd, bool __close = true);

    /**
     * @brief Copy constructor.
     * 
     * @param socket The socket to be copied from.
     * 
     * This constructor does not call socket().
     */
    basic_socket (const basic_socket& __socket);

    /**
     * @brief Destructor.
     * The destructor will call close() unless specified otherwise.
     * It never calls shutdown().
     */
    ~basic_socket();

    /**
     * @brief Bind socket to local address.
     * 
     * @param addr Pointer to buffer containing addres to bind to.
     * @param size Size of address buffer.
     * 
     * This will set the local address, which is
     * usually only important in preparation for a listen().
     */
    void
    bind (sockaddr* __addr, socklen_t __size);

    /**
     * @brief Bind socket to local address.
     * 
     * @param Address to bind to.
     * 
     * This will set the local address, which is
     * usually only important in preparation for a listen().
     */
    void
    bind (address& __addr);

    /**
     * @brief Connect socket to remote address.
     * 
     * @param addr Pointer to buffer containing address to connect to.
     * @param size Size of address buffer.
     * 
     * This will attempt to connect at a remote (listening) socket bound to the
     * specified address.
     * This is only truly meaningful for stream sockets.
     */
    void
    connect (sockaddr* __addr, socklen_t __size);

    /**
     * @brief Connect socket to remote address.
     * 
     * @param addr Address to connect to.
     * 
     * This will attempt to connect at a remote (listening) socket bound to the
     * specified address.
     * This is only truly meaningful for stream sockets.
     */
    void
    connect (address& __addr);

    /**
     * @brief Listen to incoming connections.
     * 
     * @param Backlog (can be safely left on default).
     * 
     * Puts socket in a listening state, so it can accept connections.
     * 
     * This is the opposite of connect(), so to speak, and
     * is only truly meaningful for stream sockets.
     */
    void
    listen (int __backlog = 5);

    /**
     * @brief Accept incoming connection.
     * 
     * @returns A connected socket.
     * 
     * Accepts an incoming connection on a listening socket, and returns
     * a socket connected to that peer ready for I/O.
     */
    basic_socket&
    accept ();

    /**
     * @brief Receive data on connected socket.
     */
    int
    receive (char* __s, size_t __n,
             int __flags = 0);

    /**
     * @brief Receive data on connectionless socket.
     * 
     * @param s Pointer to input buffer.
     * @param n Size of input buffer.
     * @param addr Pointer to buffer to store sender address.
     * @param size Size of address buffer.
     * @return Number of bytes received.
     * 
     * This function is only truly meaningful with connectionless sockets.
     */
    int
    receive (char* __s, size_t __n,
             sockaddr* __addr, socklen_t* __size,
             int __flags = 0);

    /**
     * @brief Receive data on connectionless socket.
     * 
     * @param s Pointer to input buffer.
     * @param n Size of input buffer.
     * @param addr Object to store sender address.
     * @return Number of bytes received.
     * 
     * This function is only truly meaningful with connectionless sockets.
     */
    int
    receive (char* __s, size_t __n, address& __addr,
             int __flags = 0);

    /**
     * @brief Receive data using msghdr structure.
     * 
     * This function uses the recvmsg() call to receive a msghdr object
     * on a connectionless socket.
     */
    int
    receive (msghdr* __msg,
             int __flags = 0);

    /**
     * @brief Send data on connected socket.
     */
    int
    send (const char* __s, size_t __n,
          int __flags = 0);

    /**
     * @brief Send data on connectionless socket.
     *
     * @param s Pointer to buffer containing data to send.
     * @param n Size of data buffer.
     * @param addr Pointer to buffer containing address to send to.
     * @param size Size of the address buffer.
     * @param flags I/O flags.
     * @return Number of bytes sent.
     * 
     * This function is only truly meaningful with connectionless sockets.
     */
    int
    send (const char* __s, size_t __n,
          sockaddr* __addr, socklen_t __size,
          int __flags = 0);

    /**
     * @brief Send data on connectionless socket.
     * 
     * @param s Pointer to buffer containing data to send.
     * @param n Size of data buffer.
     * @param addr Address to send to.
     * @param flags I/O flags.
     * @return Number of bytes sent.
     * 
     * This function is only truly meaningful with connectionless sockets.
     */
    int
    send (const char* __s, size_t __n, address& __addr,
          int __flags = 0);

    /**
     * @brief Send data on connectionless socket.
     * 
     * @param msg Pointer to msghdr buffer containing address and data.
     * @param flags I/O flags.
     * @return Number of bytes sent.
     * 
     * This function uses the sendmsg() call to send a msghdr object
     * on a connectionless socket.
     */
    int
    send (const msghdr* __msg,
          int __flags = 0);

    /**
     * @brief Set socket option.
     * 
     * @param level Option level.
     * @param option Option code.
     * @param value Pointer to buffer with option value.
     * @param size Size of option value.
     * 
     * This function sets a socket option with setsockopt().
     */
    void
    set_option (int __level, int __option,
                const char* __value, socklen_t __size);

    /**
     * @brief Get socket option.
     * 
     * @param level Option level.
     * @param option Option code.
     * @param value Pointer to buffer to store option value.
     * @param size Pointer to socklen_t to store option value size.
     * 
     * This function obtains a socket option value with getsockopt().
     */
    void
    get_option (int __level, int __option,
                char* __value, socklen_t* __size);

    /**
     * @brief Shut the socket down.
     * @param mode Mode.
     * 
     * This will close the connection with the peer for sending, receiving
     * or both.
     */
    void
    shutdown (int __mode = sockets_base::rw);

    /**
     * @brief Check if socket is connected.
     * 
     * @return true if socket is connected, false otherwise.
     */
    bool
    is_open () const;

    /**
     * @brief Peeks at input internal buffer.
     * This method is just a receive() that does not remove the data received
     * from the input buffer. A next call to receive() or peek() will result in
     * the same data.
     */
    int
    peek (int __n = 1);

    /**
     * @brief Get underlying socket descriptor.
     * 
     * @return Raw socket descriptor.
     */
    socket_type
    sd ();

  private:

    socket_type  _M_sd;

    int          _M_family;
    int          _M_style;
    int          _M_protocol;

    bool         _M_is_open;
    bool         _M_close;

  };
}

#endif	// SOCKETSTREAM_BASIC_SOCKET_H
