// -*- Socket Streams -*-

// Copyright (C) 2003 Pedro Lamarao <pedro.lamarao@izcom.com.br>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file basic_socketstream_pair.h
 * @brief Contains the pair template.
 */

#ifndef SOCKETSTREAM_BASIC_PAIR_H
#define SOCKETSTREAM_BASIC_PAIR_H 1

#include <socketstream/sockets_base.h>
#include <socketstream/basic_iosocketstream.h>

namespace socketstream {
  
  /**
   * @brief Creates a pair of connected sockets ready for I/O.
   * 
   * This is generally useful for communication between parent and child processes.
   */
  template <typename _CharT, typename _Traits>
  class basic_pair {
  public:

    //@{
    /**
     * @brief These are standard types.
     * 
     * They permit a standardized way of referring to names of
     * (or names dependant on) the template parameters, which are specific 
     * to the implementation.
     */
    typedef _CharT     char_type;
    typedef _Traits  traits_type;
    
    typedef typename traits_type::int_type  int_type;
    typedef typename traits_type::pos_type  pos_type;
    typedef typename traits_type::off_type  off_type;
    //@}
  
    typedef basic_pair<char_type, traits_type> __pair_type;
  
    /**
     * @brief Default constructor.
     */
    basic_pair ()
    : _M_which(0)
    { }
  
    /**
     * @brief Constructor.
     * 
     * Constructs a pair of the specified family and style.
     */
    basic_pair (sockets_base::family __f, sockets_base::style __s)
    : _M_which(0)
    { }
  
    /**
     * @brief Destructor.
     */
    ~basic_pair ()
    { }

    /**
     * @brief Selects the first socket.
     */
    __pair_type&
    one ()
    {
      _M_which = 0;
      return *this;
    }
  
    /**
     * @brief Selects the second socket.
     */
    __pair_type&
    two ()
    {
      _M_which = 1;
      return *this;
    }

    /**
     * @brief Returns an iosocketstream for the selected socket.
     */
    basic_iosocketstream<char_type, traits_type> &
    get_iostream ()
    {
      basic_socket __s(_M_pair[_M_which], false);
      return *new basic_iosocketstream<char_type, traits_type>(__s);
    }

    /**
     * @brief Returns the selected socket.
     */
    basic_socket&
    get_socket ()
    {
      return *new basic_socket(_M_pair[_M_which], false);
    }

  private:
    
    sockets_base::socket_type _M_pair[1];
    
    int _M_which;
  
  };
  
  typedef basic_pair<char, std::char_traits<char> > pair;
  
}

#endif // SOCKETSTREAM_BASIC_PAIR_H
