// -*- Socket Streams -*-

// Copyright (C) 2003 Pedro Lamarao <pedro.lamarao@izcom.com.br>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file sockets_base.h
 * @brief Contains the sockets_base class.
 */

#ifndef SOCKETSTREAM_SOCKETS_BASE_H
#define SOCKETSTREAM_SOCKETS_BASE_H 1

#include <sys/types.h>
#include <sys/socket.h>

/**
 * @brief Everything declared by the Socket Stream Library
 * is within namespace socketstream.
 */
namespace socketstream {

  enum __Family   { };
  enum __Style    { };
  enum __Protocol { };
  enum __ShutMode { };
  enum __Io_Flags { };
  
  inline __Io_Flags
  operator& (__Io_Flags __a, __Io_Flags __b)
  { return __Io_Flags(static_cast<int>(__a) & static_cast<int>(__b)); }
    
  inline __Io_Flags
  operator| (__Io_Flags __a, __Io_Flags __b)
  { return __Io_Flags(static_cast<int>(__a) | static_cast<int>(__b)); }

  inline __Io_Flags
  operator^ (__Io_Flags __a, __Io_Flags __b)
  { return __Io_Flags(static_cast<int>(__a) ^ static_cast<int>(__b)); }

  inline __Io_Flags
  operator&= (__Io_Flags __a, __Io_Flags __b)
  { return __a = __a & __b; }

  inline __Io_Flags
  operator|= (__Io_Flags __a, __Io_Flags __b)
  { return __a = __a | __b; }

  inline __Io_Flags
  operator^= (__Io_Flags __a, __Io_Flags __b)
  { return __a = __a ^ __b; }

  inline __Io_Flags
  operator~ (__Io_Flags __a)
  { return __Io_Flags(~static_cast<int>(__a)); }

  /**
   * @brief Base class for all socketstream specific common stuff.
   * 
   * This class is inherited by the iosocketstream classes.
   */
  class sockets_base {
  public:

    /**
     * @brief This is the socket descriptor type in the system.
     * 
     * This is usually int, but not on Windows.
     */
    typedef int socket_type;
  
    /**
     * @brief Enumerates protocol (or address) families.
     * 
     * sockets_base::family members are to be passed as the family argument to
     * basic_socket() .
     */
    typedef __Family family;

    /**
     * @brief Enumerates socket styles.
     * 
     * sockets_base::style members are to be passed as the style argument to
     * basic_socket() .
     */
    typedef __Style style;
    
    /**
     * @brief Enumerates socket protocols.
     */
    typedef __Protocol protocol;

    /**
     * @brief Enumerates shutdown modes.
     * 
     * sockets_base::shutmode members are to be passed as the mode argument to
     * basic_socket::shutdown() .
     */
    typedef __ShutMode shutmode;

    /**
     * @brief Enumerates I/O flags.
     * 
     * sockets_base::io_flags members are to be passed as the flags argument to
     * basic_socket::receive() and basic_socket::send() .
     */
    typedef __Io_Flags io_flags;

    /**
     * @brief Default values.
     * 
     * These should be set by the user at configure time, and used by
     * default constructors throughout the library.
     */
    struct def
    { 
      static sockets_base::family    family;
      static sockets_base::style     style;
      static sockets_base::protocol  protocol;
      static sockets_base::io_flags  io_flags;
    };
    
    static family local;
    static family inet;
    static family inet6;
    static family netlink;
    static family packet;
  
    static style stream;
    static style datagram;
    static style raw;
    static style rdm;
    static style seqpacket;
    
    static shutmode r;
    static shutmode w;
    static shutmode rw;
  
    static io_flags oob;
    static io_flags no_oob;
    static io_flags dontroute;
    static io_flags no_dontroute;
    static io_flags dontwait;
    static io_flags no_dontwait;
    static io_flags nosignal;
    static io_flags no_nosignal;
    static io_flags peek;
    static io_flags no_peek;
    static io_flags waitall;
    static io_flags no_waitall;
    static io_flags trunc;
    static io_flags no_trunc;
    static io_flags errqueue;
    static io_flags no_errqueue;
    
  };
}

#endif // SOCKETSTREAM_SOCKETS_BASE_H
