// -*- Socket Streams -*-

// Copyright (C) 2003 Pedro Lamarao <pedro.lamarao@izcom.com.br>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file basic_socket.cc
 * @doctodo
 */

#include <socketstream/basic_socket.h>
#include <unistd.h> // close()

namespace socketstream {

  basic_socket::basic_socket ()
  : _M_family(sockets_base::def::family), _M_style(sockets_base::def::style),
    _M_protocol(0), _M_is_open(false), _M_close(true)
  {
    _M_sd = ::socket(_M_family, _M_style, _M_protocol);
    if (_M_sd < 0)
      throw socket_exception();
  }

  basic_socket::basic_socket (int __family, int __style, int __protocol)
  : _M_family(__family), _M_style(__style), _M_protocol(__protocol),
    _M_is_open(false), _M_close(true)
  {
    _M_sd = ::socket(_M_family, _M_style, _M_protocol);
    if (_M_sd < 0)
      throw socket_exception();
  }

  basic_socket::basic_socket (socket_type __sd, bool __close)
  : _M_sd(__sd), _M_family(0), _M_style(0),
    _M_protocol(0), _M_is_open(false), _M_close(__close)
  { }

  basic_socket::basic_socket (const basic_socket& __s)
  : _M_sd(__s._M_sd), _M_family(__s._M_family), _M_style(__s._M_style),
    _M_protocol(__s._M_protocol), _M_is_open(__s._M_is_open),
    _M_close(__s._M_close)
  { }

  basic_socket::~basic_socket ()
  { // ::close() errors are irrelevant at this point, unless, perhaps, EINTR
    if (_M_is_open && _M_close)
    {
      ::close(_M_sd);
      _M_is_open = false;
    }
  }

  void
  basic_socket::bind (sockaddr* __addr, socklen_t __size)
  {
    int __r = ::bind(_M_sd, __addr, __size);
    if (__r < 0)
      throw socket_exception();
  }

  void
  basic_socket::bind (address& __addr)
  {
    sockets_base::style __s = static_cast<sockets_base::style>(_M_style);
    int __r = ::bind(_M_sd, __addr.style(__s).passive().getaddr(), __addr.getsize());
    if (__r < 0)
      throw socket_exception();
  }

  void
  basic_socket::connect (sockaddr* __addr, socklen_t __size)
  {
    // add check for family consistency
    if (::connect(_M_sd, __addr, __size))
      throw socket_exception();
    _M_is_open = true;
  }

  void
  basic_socket::connect (address& __addr)
  {
    sockets_base::style __s = static_cast<sockets_base::style>(_M_style);
    // add check for family consistency
    if (::connect(_M_sd, __addr.style(__s).getaddr(), __addr.getsize()))
      throw socket_exception();
    _M_is_open = true;
  }

  void
  basic_socket::listen (int __backlog)
  {
	  int __r = ::listen(_M_sd, __backlog);
    if (__r < 0)
      throw socket_exception();
    _M_is_open = true;
  }

  basic_socket&
  basic_socket::accept ()
  {
    socket_type __sd = ::accept(_M_sd, 0, 0);
    basic_socket *__s = new basic_socket(__sd, true);
    __s->_M_is_open = true;
    return *__s;
  }

  int
  basic_socket::receive (char* __s, size_t __n, int __flags)
  {
    int __r = ::recv(_M_sd, __s, __n, __flags);
    if (__r < 0)
      throw socket_exception();
    return __r;
  }

  int
  basic_socket::receive (char* __s, size_t __n,
                         sockaddr* __addr, socklen_t* __size, int __flags)
  {
    int __r = ::recvfrom(_M_sd, __s, __n, __flags, __addr, __size);
    if (__r < 0)
      throw socket_exception();
    return __r;
  }

  int
  basic_socket::receive (msghdr* __msg, int __flags)
  {
    int __r = ::recvmsg(_M_sd, __msg, __flags);
    if (__r < 0)
      throw socket_exception();
    return __r;
  }

  int
  basic_socket::send (const char* __s, size_t __n, int __flags)
  {
    int __r = ::send(_M_sd, __s, __n, __flags);
    if (__r < 0)
      throw socket_exception();
    return __r;
  }

  int
  basic_socket::send (const char* __s, size_t __n,
                      sockaddr* __addr, socklen_t __size, int __flags)
  {
    int __r = ::sendto(_M_sd, __s, __n, __flags, __addr, __size);
    if (__r < 0)
      throw socket_exception();
    return __r;
  }

  int
  basic_socket::send (const msghdr* __msg, int __flags)
  {
    int __r = ::sendmsg(_M_sd, __msg, __flags);
    if (__r < 0)
      throw socket_exception();
    return __r;
  }

  void
  basic_socket::get_option (int __level, int __option, char* __value, socklen_t* __size)
  {
    if (::getsockopt(_M_sd, __level, __option, __value, __size))
      throw socket_exception();
  }

  void
  basic_socket::set_option (int __level, int __option, const char* __value, socklen_t __size)
  {
    if (::setsockopt(_M_sd, __level, __option, __value, __size))
      throw socket_exception();
  }

  void
  basic_socket::shutdown (int __mode)
  {
    int __m = __mode;
    if (::shutdown(_M_sd, __m))
      throw socket_exception();
  }

  bool
  basic_socket::is_open () const
  {
    return _M_is_open;
  }

  int
  basic_socket::peek (int __n)
  {
    char __c[__n];
    int __r = recv(_M_sd, &__c, __n, MSG_PEEK);
    if (__r < 0)
      throw socket_exception();
    return __r;
  }
}
